@@ -20,6 +20,7 @@ from typing import Any, Iterator, Iterable, Mapping, Optional, Text, Type
 
 from .exceptions import ObjectAlreadyExists
 from .utils import intersection, OrderedDefaultDict
+from .enum import DiffSyncActions
 
 
 class Diff:
@@ -55,7 +56,7 @@ class Diff:
         """
         # Note that element.name is usually a DiffSyncModel.shortname() -- i.e., NOT guaranteed globally unique!!
         if element.name in self.children[element.type]:
-            raise ObjectAlreadyExists(f"Already storing a {element.type} named {element.name}")
+            raise ObjectAlreadyExists(f"Already storing a {element.type} named {element.name}", element)
 
         self.children[element.type][element.name] = element
 
@@ -105,9 +106,9 @@ class Diff:
     def summary(self) -> Mapping[Text, int]:
         """Build a dict summary of this Diff and its child DiffElements."""
         summary = {
-            "create": 0,
-            "update": 0,
-            "delete": 0,
+            DiffSyncActions.CREATE.value: 0,
+            DiffSyncActions.UPDATE.value: 0,
+            DiffSyncActions.DELETE.value: 0,
             "no-change": 0,
         }
         for child in self.get_children():
@@ -220,22 +221,22 @@ class DiffElement:  # pylint: disable=too-many-instance-attributes
         return total
 
     @property
-    def action(self) -> Optional[Text]:
+    def action(self) -> Optional[DiffSyncActions]:
         """Action, if any, that should be taken to remediate the diffs described by this element.
 
         Returns:
-            str: "create", "update", "delete", or None
+            DiffSyncActions ("create", "update", "delete", or None)
         """
         if self.source_attrs is not None and self.dest_attrs is None:
-            return "create"
+            return DiffSyncActions.CREATE
         if self.source_attrs is None and self.dest_attrs is not None:
-            return "delete"
+            return DiffSyncActions.DELETE
         if (
             self.source_attrs is not None
             and self.dest_attrs is not None
             and any(self.source_attrs[attr_key] != self.dest_attrs[attr_key] for attr_key in self.get_attrs_keys())
         ):
-            return "update"
+            return DiffSyncActions.UPDATE
 
         return None
 
@@ -328,13 +329,13 @@ class DiffElement:  # pylint: disable=too-many-instance-attributes
     def summary(self) -> Mapping[Text, int]:
         """Build a summary of this DiffElement and its children."""
         summary = {
-            "create": 0,
-            "update": 0,
-            "delete": 0,
+            DiffSyncActions.CREATE.value: 0,
+            DiffSyncActions.UPDATE.value: 0,
+            DiffSyncActions.DELETE.value: 0,
             "no-change": 0,
         }
         if self.action:
-            summary[self.action] += 1
+            summary[self.action.value] += 1
         else:
             summary["no-change"] += 1
         child_summary = self.child_diff.summary()
